@@ -2,26 +2,34 @@
 
 package Net::NBsocket;
 use strict;
-#use diagnostics;
+use diagnostics;
 
 use vars qw(
 	$VERSION @ISA @EXPORT_OK $UDP $TCP
 );
 use POSIX;
 use Socket;
-use AutoLoader 'AUTOLOAD';
+#use AutoLoader 'AUTOLOAD';
 require Exporter;
 @ISA = qw(Exporter);
 
-$VERSION = do { my @r = (q$Revision: 0.06 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 0.08 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 @EXPORT_OK = qw(
-        open_udpNB
+	open_UDP
+	open_udpNB
+	bind2pp
+	open_Listen
 	open_listenNB
 	connect_NB
+	accept_Blk
 	accept_NB
 	set_NB
 	set_so_linger
+	inet_aton
+	inet_ntoa
+	sockaddr_in
+	sockaddr_un
 );
 
 # used a lot, create once per session
@@ -30,8 +38,8 @@ $TCP = getprotobyname('tcp');
 
 sub DESTROY {};
 
-1;
-__END__
+#1;
+#__END__
 
 =head1 NAME
 
@@ -40,21 +48,40 @@ Net::NBsocket -- Non-Blocking Sockets
 =head1 SYNOPSIS
 
   use Net::NBsocket qw(
-        open_udpNB
+	open_UDP
+	open_udpNB
+	bind2pp
+	open_Listen
 	open_listenNB
+	connectBlk
 	connect_NB
+	accept_Blk
 	accept_NB
 	set_NB
 	set_so_linger
+	inet_aton
+	inet_ntoa
+	sockaddr_in
+	sockaddr_un
   );
 
+  $sock = open_UDP();
   $sock = open_udpNB();
+  $sock = bind2pp($sock,$port_path,$netaddr);
+  $listener = open_Listen($port_path,$netaddr);
   $listener = open_listenNB($port_path,$netaddr);
-  $rv = set_sockNB(*SOCK);
+  $rv = set_NB(*SOCK);
   $rv = set_so_linger(*HANDLE,$seconds);
+  $client = connectBlk($port_path,$netaddr);
   $client = connect_NB($port_path,$netaddr);
+  ($sock,$netaddr) = accept_Blk(*SERVER);
   ($sock,$netaddr) = accept_NB(*SERVER);
-    
+  $netaddr = inet_aton($dot_quad);
+  $dot_quad = inet_ntoa($netaddr);
+  $sin = sockaddr_in($port,$netaddr);
+  ($port,$netaddr) = sockaddr_in($sin);
+  $sun = sockaddr_un($path);
+  ($path) = sockaddr_un($sun);
 
 =head1 DESCRIPTION
 
@@ -63,9 +90,27 @@ sockets of various flavors;
 
 =over 4
 
+=item * $netaddr = inet_aton($dot_quad);
+
+=item * $dot_quad = inet_ntoa($netaddr);
+
+=item * $sin = sockaddr_in($port,$netaddr);
+
+=item * ($port,$netaddr) = sockaddr_in($sin);
+
+=item * $sun = sockaddr_un($path);
+
+=item * ($path) = sockaddr_un($sun);
+
+All above exported from B<Socket> in the EXPORT_OK array.
+
+=item * $sock = open_UDP();
+
+Open an unbound UDP socket as below.
+
 =item * $sock = open_udpNB();
 
-Open and return a non-blocking UDP socket object
+Open and return an unbound  non-blocking UDP socket object
 
   input:	none
   returns:	pointer to socket object
@@ -73,16 +118,52 @@ Open and return a non-blocking UDP socket object
 
 =cut
 
-sub open_udpNB {
-  my $flags;
+sub open_UDP {
   local *USOCK;
-  return undef unless socket(USOCK,PF_INET,SOCK_DGRAM,$UDP);
-  return *USOCK if set_NB(*USOCK);
-  close USOCK;
+  return *USOCK if socket(USOCK,PF_INET,SOCK_DGRAM,$UDP);
+  close USOCK if scalar *USOCK;
   return undef;
 }
 
-=item * $listener = open_listenNB($port_path);
+sub open_udpNB {
+  local *USOCK;
+  return *USOCK if socket(USOCK,PF_INET,SOCK_DGRAM,$UDP) && set_NB(*USOCK);
+  close USOCK if scalar *USOCK;
+  return undef;
+}
+
+=item * $sock = bind2pp($sock,$port_path,$netaddr);
+
+Bind to $port_path and an optional IPv4 bind address as returned by inet_aton
+(defaults to INADDR_ANY).
+
+  input:	port or unix domain socket path,
+		[optional] bind address
+  returns:	socket on sucess, else undef;
+
+=cut
+
+sub bind2pp {
+  my ($sock,$port_path,$addr) = @_;;
+  $addr = INADDR_ANY unless $addr;
+  my $path = ($port_path && $port_path =~ /[\D\s]/) ? $port_path : undef;
+  my $ok;
+  if ($path) {
+    unlink $path if -e $path && -S $path;
+    $ok = bind($sock,sockaddr_un($path));
+  } else {
+    $ok = bind($sock,sockaddr_in($port_path,$addr));
+  }
+  return $sock if $ok;
+  close $sock;
+  return undef;
+}
+
+=item * $listener = open_Listen($port_path,$netaddr);
+
+Open a blocking TCP listner as below.
+
+=item * $listener = open_listenNB($port_path,$netaddr);
 
 Open and return a non-blocking TCP listener bound to $port_path and an
 optional IPv4 bind address as returned by inet_aton 
@@ -99,32 +180,31 @@ The user must set the appropriate UMASK prior to calling this routine.
 
 =cut
 
-sub open_listenNB {
+sub open_Listen {
   my ($port_path,$addr) = @_;;
   local *LSOCK;
-  $addr = INADDR_ANY unless $addr;
-
-  my $path = ($port_path =~ /\D/) ? $port_path : undef;
-  if ($path) {
+  if ($port_path && $port_path =~ /[\D\s]/) {
     return undef unless socket(LSOCK,PF_UNIX,SOCK_STREAM,0);
   } else {
     return undef unless socket(LSOCK,PF_INET,SOCK_STREAM,$TCP);
   }
-  my $ok = setsockopt(LSOCK,SOL_SOCKET,SO_REUSEADDR,pack("l", 1));
-  if ($path) {
-    unlink $path if -e $path && -S $path;
-    ($ok = bind(LSOCK,sockaddr_un($path))) if $ok;
-  } else {
-    ($ok = bind(LSOCK,sockaddr_in($port_path,$addr))) if $ok;
-  }
-  return *LSOCK if $ok &&
-	listen(LSOCK,SOMAXCONN) &&
-	set_NB(*LSOCK);
-  close LSOCK;
+  my $sockok = setsockopt(LSOCK,SOL_SOCKET,SO_REUSEADDR,pack("l", 1));
+# function returns LSOCK if success
+  $sockok = bind2pp(*LSOCK,$port_path,$addr) if $sockok;
+  return $sockok if $sockok &&
+        listen($sockok,SOMAXCONN);
+  close $sockok;
   return undef;
 }
 
-=item * $rv = set_sockNB(*SOCK);
+sub open_listenNB {
+  my $lsock = &open_Listen;
+  return $lsock if $lsock && set_NB($lsock);
+  close $lsock if $lsock;
+  return undef;
+}
+
+=item * $rv = set_NB(*SOCK);
 
 Set a socket to Non-Blocking mode
 
@@ -154,6 +234,10 @@ sub set_so_linger {
   setsockopt($FH,SOL_SOCKET,SO_LINGER,pack("ll",1,$sec));
 }
 
+=item * $client = connectBlk($port_path,$netaddr);
+
+Begin a blocking TCP connection as below.
+
 =item * $client = connect_NB($port_path,$netaddr);
 
 Begin a non-blocking TCP connection to the host designated by $netaddr on
@@ -168,8 +252,18 @@ $netaddr is unused for unix domain sockets.
 
 =cut
 
-sub connect_NB {
-  my($port_path,$netaddr) = @_;
+sub connectBlk {
+  unshift @_,1;
+  goto &_connect;
+}
+
+sub  connect_NB {
+  unshift @_,0;
+  goto &_connect;
+}
+
+sub _connect {
+  my($block,$port_path,$netaddr) = @_;
   local *CSOCK;
   my $daddr;
   if ($port_path =~ /\D/) {
@@ -179,13 +273,17 @@ sub connect_NB {
     $daddr = sockaddr_in($port_path,$netaddr);
     return undef unless $daddr && socket(CSOCK,PF_INET,SOCK_STREAM,$TCP);
   }
-  if (set_NB(*CSOCK)) {
+  if ($block || set_NB(*CSOCK)) {
     return *CSOCK if connect(CSOCK,$daddr) || $! == EINPROGRESS;
   }
   close CSOCK;
   return undef;
 }
 
+=item * ($sock,$netaddr) = accept_Blk(*SERVER);
+
+Accept a connection and return a BLOCKING socket as below.
+
 =item * ($sock,$netaddr) = accept_NB(*SERVER);
 
 Accept a connection from a remote client, return a non-blocking socket
@@ -203,7 +301,17 @@ the unix domain socket path if PF_INET or PF_UNIX respectively.
 =cut
 
 sub accept_NB {
-  my $server = shift;
+  unshift @_,0;
+  goto &_accept;
+}
+
+sub accept_Blk {
+  unshift @_,1;
+  goto &_accept;
+}
+
+sub _accept {
+  my($block,$server) = @_;
   local *CLONE;
   my $paddr = accept(CLONE,$server);
   return () unless $paddr;		# attempted accept with no client
@@ -212,7 +320,7 @@ sub accept_NB {
     $netaddr = sockaddr_un($paddr);
   }
   return (*CLONE,$netaddr)
-	if $paddr && $netaddr && set_NB(*CLONE);
+	if $paddr && $netaddr && ($block || set_NB(*CLONE));
   close CLONE;
   return ();
 }
@@ -224,12 +332,21 @@ sub accept_NB {
 
 =head1 EXPORT_OK
 
+	open_UDP
         open_udpNB
+	bind2pp
+	open_Listen
 	open_listenNB
+	connectBlk
 	connect_NB
+	accept_Blk
 	accept_NB
 	set_NB
 	set_so_linger
+	inet_aton
+	inet_ntoa
+	sockaddr_in
+	sockaddr_un
 
 =head1 AUTHOR
 
@@ -237,7 +354,7 @@ Michael Robinton, michael@bizsystems.com
 
 =head1 COPYRIGHT
 
-Copyright 2004, Michael Robinton & BizSystems
+Copyright 2004 - 2006, Michael Robinton & BizSystems
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or 
@@ -252,6 +369,10 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+=head1 SEE ALSO
+
+L<POSIX>, L<Socket>
+
 =cut
 
 1;
